package be.iminds.ilabt.jfed.experimenter_gui.editor.ontology;

import be.iminds.ilabt.jfed.experimenter_gui.editor.ModelRspecEditor;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ontology.controller.NodePropertyOverview;
import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi.ComponentManagerInfo;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.FXUtils;
import be.iminds.ilabt.jfed.rspec.model.ModelRspec;
import be.iminds.ilabt.jfed.rspec.model.RspecNode;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.geometry.Rectangle2D;
import javafx.scene.Scene;
import javafx.scene.layout.StackPane;
import javafx.stage.Screen;
import javafx.stage.Stage;
import javafx.stage.Window;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * User: twalcari Date: 4/10/2014 Time: 11:55
 */
public class OntologyAdaptivePropertiesDialog extends StackPane {
    private static final Logger LOG = LoggerFactory.getLogger(OntologyAdaptivePropertiesDialog.class);

    private final RspecNode node;
    private final ModelRspec model;

    private Map<String, ComponentManagerInfo> componentManagerInfoMap = null;
    private Map<String, String> testbedInfo = null;
    private Map<String, String> reversedTestbedInfo = null;
    private Map<String, String> nodeToTestbed = null;
    private Map<String, String> fakeToRealUrn = null;

    private List<String> criteriaData;
    private Map<String, List<String>> criteria;
    private Map<String, String> reversedCriteria;
    private NodePropertyOverview controller;

    private OntologyAdaptivePropertiesDialog(ModelRspec model, RspecNode node) {
        assert (model != null);
        assert (node != null);
        this.model = model;
        this.node = node;

        criteriaData = new ArrayList<>();
        criteria = new HashMap<>();
        criteriaData.add("memoryCapacity"); //name in ontology
        criteriaData.add("double"); //type
        criteriaData.add("[GB]"); //used to show what user has to fill in
        criteria.put("Disk space", criteriaData); //used to show a proper name for the user
        criteriaData = new ArrayList<>();
        criteriaData.add("ramMemory");
        criteriaData.add("double");
        criteriaData.add("[MB]");
        criteria.put("RAM memory", criteriaData);
        criteriaData = new ArrayList<>();
        criteriaData.add("cpu");
        criteriaData.add(null);
        criteriaData.add(null);
        criteria.put("CPU", criteriaData);
        criteriaData = new ArrayList<>();
        criteriaData.add("os");
        criteriaData.add(null);
        criteriaData.add(null);
        criteria.put("Operating system", criteriaData);
        criteriaData = new ArrayList<>();
        criteriaData.add("sliverType");
        criteriaData.add(null);
        criteriaData.add(null);
        criteria.put("Slivertype", criteriaData);
        criteriaData = new ArrayList<>();
        criteriaData.add("lat");
        criteriaData.add("double");
        criteriaData.add("[-90, 90]");
        criteria.put("Latitude", criteriaData);
        criteriaData = new ArrayList<>();
        criteriaData.add("long");
        criteriaData.add("double");
        criteriaData.add("[-180, 180]");
        criteria.put("Longitude", criteriaData);
        criteriaData = new ArrayList<>();
        criteriaData.add("testbed");
        criteriaData.add(null);
        criteriaData.add(null);
        criteria.put("Testbed", criteriaData);
        initializeReversedCrit();

        final NodePropertyOverview nodePropertyOverview = new NodePropertyOverview();
        getChildren().add(nodePropertyOverview);
        nodePropertyOverview.setNameTextFieldText(node.getClientId());
        nodePropertyOverview.setNode(node);
        nodePropertyOverview.setModel(model);
        nodePropertyOverview.setMainApp(this);


        testbedInfo = new HashMap<>();
        reversedTestbedInfo = new HashMap<>();
        nodeToTestbed = new HashMap<>();
        fakeToRealUrn = new HashMap<>();
    }

    public static void showPropertiesDialog(Window parent, ModelRspec model, RspecNode node) {

        OntologyAdaptivePropertiesDialog dialog = new OntologyAdaptivePropertiesDialog(model, node);
        Scene scene = new Scene(dialog, 600, 600);

        Stage dialogStage = new Stage();
        dialogStage.setScene(scene);
        Rectangle2D primaryScreenBounds = Screen.getPrimary().getVisualBounds();
        dialogStage.setX(primaryScreenBounds.getMinX() + primaryScreenBounds.getWidth() / 6);
        dialogStage.setY(primaryScreenBounds.getMinY() + primaryScreenBounds.getHeight() / 4);

        dialogStage.setTitle("Properties of " + node.getClientId());
        dialogStage.setResizable(true);
        FXUtils.centerToOwner(parent, dialogStage);

        dialogStage.showAndWait();
    }


    @FXML
    @SuppressWarnings("unchecked")
    public void onSaveAction(ActionEvent actionEvent) {

        if (!isValidNodeName(controller.getNameTextField().getText())) {
            JFDialogs.create()
                    .owner(this)
                    .message("A node name may only contain alphanumerical characters, and is maximum " + ModelRspecEditor.MAX_NODE_NAME_LENGTH + " characters long.")
                    .masthead("The entered node name is invalid.")
                    .showError();
            return;
        }

        RspecNode currentNode = this.node;
        currentNode.setClientId(controller.getNameTextField().getText());

        Stage stage = (Stage) controller.getSaveButton().getScene().getWindow();
        stage.close();
    }

//    private void setSceneEh() {
//        this.getScene().setOnMouseClicked(t -> {
//            CustomControlView ccv = (CustomControlView) controller.getToggleGroup().getSelectedToggle();
//            if (ccv != null) {
//                ccv.performSave();
//            }
//        });
//    }

    private int getMaximumNodeNameLength() {
        return 10;
    }

    private boolean isValidNodeName(String text) {
        if (text.length() > getMaximumNodeNameLength()) {
            return false;
        }

        for (char c : text.toCharArray()) {
            boolean validChar = (c >= 'a' && c <= 'z')
                    || (c >= 'A' && c <= 'Z')
                    || (c >= '0' && c <= '9');

            if (!validChar) {
                return false;
            }
        }

        return true;
    }

    @FXML
    public void onCancelAction(ActionEvent actionEvent) {
        Stage stage = (Stage) controller.getCancelButton().getScene().getWindow();
        stage.close();
    }

    public Map<String, List<String>> getCriteria() {
        return criteria;
    }

    public Map<String, String> getReversedCriteria() {
        return reversedCriteria;
    }

    public Map<String, ComponentManagerInfo> getComponentManagerInfoMap() {
        return componentManagerInfoMap;
    }

    public Map<String, String> getTestbedInfo() {
        return testbedInfo;
    }

    public Map<String, String> getReversedTestbedInfo() {
        return reversedTestbedInfo;
    }

    public Map<String, String> getNodeToTestbed() {
        return nodeToTestbed;
    }

    public void setNodeToTestbed(Map<String, String> nodeToTestbed) {
        this.nodeToTestbed = nodeToTestbed;
    }

    public Map<String, String> getFakeToRealUrn() {
        return fakeToRealUrn;
    }

    private void initializeReversedCrit() {
        reversedCriteria = new HashMap<>();
        for (Map.Entry<String, List<String>> entry : criteria.entrySet()) {
            String key = entry.getKey();
            String value = criteria.get(key).get(0);
            reversedCriteria.put(value, key);
        }
    }
}
