/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package be.iminds.ilabt.jfed.experimenter_gui.editor.ontology.view;

import be.iminds.ilabt.jfed.util.ExecuteOnNotNull;
import javafx.beans.binding.StringBinding;
import javafx.beans.property.SimpleObjectProperty;
import javafx.collections.FXCollections;
import javafx.geometry.Insets;
import javafx.geometry.Point2D;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.ToggleButton;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.util.StringConverter;

import javax.annotation.Nullable;
import java.util.Collection;

/**
 * @author Patrick
 */
public abstract class ListFilterView<T> extends FilterCriteriaView {

    private SimpleObjectProperty<T> value;

    private final String title;
    //popup fileds
    private final Stage stage;
    private final ComboBox<T> listView;
    private final Button okButton;

    public ListFilterView(String title, Collection<T> values, @Nullable StringConverter<T> converter) {
        this.title = title;

        //initialize Popup
        stage = new Stage(StageStyle.TRANSPARENT);
        StackPane layout = new StackPane();
        VBox popupVBox = new VBox(5);
        listView = new ComboBox<>(FXCollections.observableArrayList(values));
        listView.setConverter(converter);

        okButton = new Button("OK");
        okButton.setOnAction(event -> {
            hideStage();
        });

        HBox hBox = new HBox();
        hBox.getChildren().add(listView);

        HBox hBox2 = new HBox();
        hBox2.setAlignment(Pos.CENTER_RIGHT);
        hBox2.getChildren().addAll(okButton);
        popupVBox.setPadding(new Insets(5));
        popupVBox.getChildren().addAll(hBox, hBox2);
        popupVBox.setStyle("-fx-border-color: grey;"
                + "-fx-border-width: 2px;");
        layout.getChildren().addAll(popupVBox);

        Scene scene = new Scene(layout, Color.LIGHTGREY);
        stage.setScene(scene);
        stage.setResizable(false);

        this.setOnAction(event -> {
            ToggleButton button = (ToggleButton) event.getSource();
            if (stage.isShowing()) {
                stage.hide();
            } else {
                Scene scene1 = button.getScene();
                final Point2D windowCoord = new Point2D(scene1.getWindow().getX(), scene1.getWindow().getY());
                final Point2D sceneCoord = new Point2D(scene1.getX(), scene1.getY());
                final Point2D nodeCoord = button.localToScene(0.0, 0.0);
                stage.setX(windowCoord.getX() + sceneCoord.getX() + nodeCoord.getX());
                stage.setY(windowCoord.getY() + sceneCoord.getY() + nodeCoord.getY() + button.getHeight());
                stage.show();
                okButton.requestFocus();
            }
        });
//        this.setOnMousePressed(event -> {
//            ToggleButton button = (ToggleButton) event.getSource();
//            CustomControlView ccv = (CustomControlView) button.getToggleGroup().getSelectedToggle();
//            if (ccv != null && ccv != button) {
//                ccv.hideStage();
//                ccv.setSelected(false);
//            }
//        });

        //bind text to underlying values
        textProperty().bind(new StringBinding() {
            {
                bind(listView.getSelectionModel().selectedItemProperty());
            }

            @Override
            protected String computeValue() {
                if (listView.getSelectionModel().getSelectedItem() != null) {
                    if (listView.getConverter() != null)
                        return title + " = " + listView.getConverter().toString(listView.getSelectionModel().getSelectedItem());
                    else
                        return title + " = " + listView.getSelectionModel().getSelectedItem().toString();
                } else {
                    return title;
                }
            }
        });

        new ExecuteOnNotNull<Scene>(sceneProperty()){
            @Override
            public void run() {
                stage.initOwner(getScene().getWindow());
            }
        };


    }


    public void hideStage() {
        stage.hide();
    }

    public String getTitle() {
        return title;
    }

    @Override
    public String getName() {
        return title;
    }

    @Override
    public boolean isValid() {
        return !listView.getSelectionModel().isEmpty();
    }
}
