package be.iminds.ilabt.jfed.experimenter_gui.login;

import be.iminds.ilabt.jfed.experimenter_gui.debug.LogViewer;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.StageUtils;
import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.util.JFedUtils;
import com.google.common.io.Files;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.fxml.FXML;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyCodeCombination;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.web.WebView;
import javafx.stage.Stage;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.cookie.BasicClientCookie;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Provider;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.net.HttpCookie;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

/**
 * User: twalcari
 * Date: 8/11/2015
 * Time: 15:06
 */
public class URLLogin extends BorderPane {
    private static final Logger LOG = LoggerFactory.getLogger(URLLogin.class);

    public final static String GENI_DOWNLOAD_URL = "https://portal.geni.net/secure/kmcert.php";
    public static final String LOGIN_CERTS_DIR = "login-certs";
    //public final static String WALL2_DOWNLOAD_URL = "https://authority.ilabt.iminds.be/";

    private final Provider<LogViewer> logviewerProvider;


    private final Stage stage = new Stage();

    @FXML
    private WebView loginWebView;

    @FXML
    private HBox loadingHBox;
    @FXML
    private ProgressBar loadingProgressBar;

    @FXML
    private Label loadingLabel;


    private final java.net.CookieManager manager = new java.net.CookieManager();

    private String result = null;
    private static final Charset UTF8_CHARSET = Charset.forName("UTF-8");


    public URLLogin(String initialUrl, Provider<LogViewer> logviewerProvider) {
        this.logviewerProvider = logviewerProvider;

        stage.setTitle("JFed login: fetch certificate");
        StageUtils.setJFedStageIcons(stage);


        FXMLUtil.injectFXML(this);

        java.net.CookieHandler.setDefault(manager);

        loadingProgressBar.progressProperty().bind(
                loginWebView.getEngine().getLoadWorker().progressProperty());

        loadingHBox.visibleProperty().bind(
                loginWebView.getEngine().getLoadWorker().runningProperty());

        loginWebView.getEngine().setConfirmHandler(param -> new Alert(Alert.AlertType.CONFIRMATION, param)
                .showAndWait()
                .filter(buttonType -> buttonType == ButtonType.YES).isPresent());

        loginWebView.getEngine().setOnAlert(text ->
                new Alert(Alert.AlertType.WARNING, text.toString()).show());

        loginWebView.getEngine().setOnError(error -> {
            LOG.info("Received ERROR from webview: {}", error.getMessage(), error.getException());

            JFDialogs.create()
                    .title("Error during web-login")
                    .showException(error.getException());

        });

        loginWebView.getEngine().getLoadWorker().exceptionProperty().addListener(observable1 -> {
            final Throwable exception = loginWebView.getEngine().getLoadWorker().getException();
            LOG.warn("Login webview has error: {}",
                    exception.getMessage(),
                    exception);

            JFDialogs.create()
                    .title("Error in webpage")
                    .message("jFed encountered an error while loading the login-webpage: " + exception.getMessage())
                    .showException(exception);
        });


        loadingLabel.textProperty().bind(
                loginWebView.getEngine().getLoadWorker().messageProperty());

        loginWebView.getEngine().documentProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue != null) {
                String text = (String) loginWebView.getEngine().executeScript("document.documentElement.outerText");
                if (text != null && text.startsWith("-----BEGIN")) {
                    processCertificate(text);
                }
            }
        });


        loginWebView.getEngine().locationProperty().addListener(new ChangeListener<String>() {
            @Override
            public void changed(ObservableValue<? extends String> observable, String oldValue, String newValue) {
                if (newValue.equals(GENI_DOWNLOAD_URL)) {
                    String geniCert = downloadGeniCert();
                    if (geniCert != null)
                        processCertificate(geniCert);
                    else {
                        LOG.warn("Downloading GENI certificate failed");
                    }
                }
            }
        });


        loginWebView.getEngine().load(initialUrl);
    }

    @Nullable
    private String downloadGeniCert() {
        try {
            DefaultHttpClient client = new DefaultHttpClient();
            client.setCookieStore(createApacheCookieStore());

            HttpPost downloadPost = new HttpPost(GENI_DOWNLOAD_URL);
            List<NameValuePair> nvps = new ArrayList<>();
            nvps.add(new BasicNameValuePair("download", "y"));
            downloadPost.setEntity(new UrlEncodedFormEntity(nvps));

            HttpResponse response = client.execute(downloadPost);


            return EntityUtils.toString(response.getEntity());


        } catch (IOException e) {
            LOG.error("Error while downloading GENI certificate", e);
            return null;
        }
    }

    @Nonnull
    private BasicCookieStore createApacheCookieStore() {
        BasicCookieStore cookieStore = new BasicCookieStore();
        for (HttpCookie httpCookie : manager.getCookieStore().getCookies()) {
            final BasicClientCookie cookie =
                    new BasicClientCookie(httpCookie.getName(), httpCookie.getValue());
            cookie.setDomain(httpCookie.getDomain());
            cookie.setPath(httpCookie.getPath());
            cookie.setComment(httpCookie.getComment());
            cookie.setSecure(httpCookie.getSecure());
            cookie.setVersion(httpCookie.getVersion());

            cookieStore.addCookie(cookie);
        }
        return cookieStore;
    }

    private void processCertificate(String certificate) {
        result = certificate;
        stage.close();
    }

    public static File createFileForLoginCertificate(String certificate){
        String certmd5 = DigestUtils.md5Hex(certificate);

        File certDir = getLoginCertificateCacheDir();
        if (!certDir.exists())
            certDir.mkdirs();

        return getLoginCertificateFile(certmd5);
    }

    public static boolean saveCertificate(File certFile, String certificate){
        try (BufferedWriter writer = Files.newWriter(certFile, UTF8_CHARSET)) {
            writer.write(certificate);
        } catch (IOException e) {
            LOG.error("Could not write certificate to file {}", certFile, e);
            return false;
        }
        return true;
    }
                   /*
    @Nullable
    public static File saveCertificate(String certificate) {
        File certFile = createFileForLoginCertificate(certificate);
        try (BufferedWriter writer = Files.newWriter(certFile, UTF8_CHARSET)) {
            writer.write(certificate);
        } catch (IOException e) {
            LOG.error("Could not write certificate to file {}", certFile, e);
            return null;
        }
        return certFile;
    }   */

    @Nonnull
    private static File getLoginCertificateCacheDir() {
        return new File(JFedUtils.getUserDataDirectoryFile(), LOGIN_CERTS_DIR);
    }

    public static File getLoginCertificateFile(String certMd5) {
        return new File(getLoginCertificateCacheDir(), certMd5 + ".pem");
    }

    public String showDialog() {


        final Scene scene = new Scene(this);
        scene.getAccelerators().put(
                new KeyCodeCombination(KeyCode.F10, KeyCodeCombination.CONTROL_DOWN),
                () -> logviewerProvider.get().show()
        );
        stage.setScene(scene);


        stage.showAndWait();

        return result;
    }
}
