package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.highlevel.controller.JFedHighLevelException;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api.RestApi;
import be.iminds.ilabt.jfed.lowlevel.api.SlaCollector;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedConnectionProvider;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.JavaFXLogger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Date;

/**
 * User: twalcari
 * Date: 10/29/2015
 * Time: 16:29
 */
public class RegisterSliversToSLACollectorTask extends SlaTask {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(RegisterSliversTask.class);

    @Nonnull
    private final Slice slice;
    @Nonnull
    private final Collection<GeniUrn> createdSliverUrns;
    @Nonnull
    private final SfaAuthority auth;


    public RegisterSliversToSLACollectorTask(@Nonnull Slice slice, @Nonnull SfaAuthority auth,
                                             @Nonnull Collection<GeniUrn> createdSliverUrns,
                                             JavaFXLogger logger, JFedPreferences jFedPreferences, AuthorityList authorityList,
                                             JFedConnectionProvider connectionProvider, GeniUserProvider geniUserProvider) {
        super("Registering slivers created @ " + auth.getUrnString() + " to SLA Collector", logger, jFedPreferences, authorityList, connectionProvider, geniUserProvider);

        this.slice = slice;
        this.createdSliverUrns = createdSliverUrns;
        this.auth = auth;

    }

    @Override
    protected void doSlaTask(TaskExecution taskExecution, SlaCollector slaCollector) throws JFedException, InterruptedException {
        RestApi.RestReply<String> result = slaCollector.registerSLA(createConnection(),
                slice.getUuid(),
                auth.getUrn(), Date.from(slice.getExpirationDate()),
                slice.getUrn(),
                geniUserProvider.getLoggedInGeniUser().getUserUrn(),
                createdSliverUrns);

        if (result.getHttpCallDetails().getResultHttpStatusCode() != 200)
            throw new JFedHighLevelException("Error while registering slivers to SLA collector");


        result = slaCollector.registerStart(createConnection(), slice.getUuid(), auth.getUrn());


        if (result.getHttpCallDetails().getResultHttpStatusCode() /100 != 2)
            throw new JFedHighLevelException("Error while registering start of slivers at " + auth.getName() + " to SLA collector");


    }

    public SfaAuthority getAuthority() {
        return auth;
    }
}
