package be.iminds.ilabt.jfed.highlevel.util;

import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticAggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.util.JavaFXLogger;

import javax.inject.Inject;
import javax.inject.Singleton;

/**
 * User: twalcari
 * Date: 6/16/2015
 * Time: 15:39
 */
@Singleton
public class AggregateManagerWrapperFactory {
    private final AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory
            automaticAggregateManagerWrapperFactory;
    private final JavaFXLogger logger;


    @Inject
    public AggregateManagerWrapperFactory(AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory,
                                          JavaFXLogger logger) {
        this.automaticAggregateManagerWrapperFactory = automaticAggregateManagerWrapperFactory;
        this.logger = logger;
    }


    public AggregateManagerWrapper getAggregateManagerWrapper(SfaAuthority amAuth){
        return automaticAggregateManagerWrapperFactory.create(amAuth);
    }


    /**
     * this is public, anyone who wants to use this directly is welcome.
     *
     * @param amAuth        the Aggregate Manager to connect to
     * @param taskExecution the TaskExecution object that needs to receive copies of all calls. May be null.
     */
    public AggregateManagerWrapper getAggregateManagerWrapper(TaskExecution taskExecution, SfaAuthority amAuth) {
        JavaFXLogger wrappedLogger;

        if (taskExecution != null) {
            wrappedLogger = JavaFXLogger.wrap(logger, taskExecution);
        } else {
            wrappedLogger = logger;
        }

        return automaticAggregateManagerWrapperFactory.create(wrappedLogger, amAuth);
    }
}
