package be.iminds.ilabt.jfed.lowlevel;

/**
 * RetrySettings
 */
public class RetrySettings {
    private final int timeoutMaxRetries;
    private final long timeoutMsBeforeRetry;

    private final int busyMaxRetries;
    private final long busyMsBeforeRetry;

    private final int possibleTemporaryErrorMaxRetries;
    private final long possibleTemporaryErrorMsBeforeRetry;

    public static RetrySettings getNoRetrySettings() { return new RetrySettings(0, 0, 0, 0, 0, 0); }

    public RetrySettings() {
        this.timeoutMaxRetries = 3;
        this.timeoutMsBeforeRetry = 5000;

        this.busyMaxRetries = 100; //changed to 100, see FEDIBBTDEV-894
                                   //was 25: at least 2 minutes (24 * 5s wait + calls themselves)
        this.busyMsBeforeRetry = 5000;

        this.possibleTemporaryErrorMaxRetries = 1; //only retry once
        this.possibleTemporaryErrorMsBeforeRetry = 1000; //wait 1 second
    }

    public RetrySettings(int timeoutMaxRetries, long timeoutMsBeforeRetry,
                         int busyMaxRetries, long busyMsBeforeRetry,
                         int possibleTemporaryErrorMaxRetries, long possibleTemporaryErrorMsBeforeRetry) {
        this.timeoutMaxRetries = timeoutMaxRetries;
        this.timeoutMsBeforeRetry = timeoutMsBeforeRetry;
        this.busyMaxRetries = busyMaxRetries;
        this.busyMsBeforeRetry = busyMsBeforeRetry;

        this.possibleTemporaryErrorMaxRetries = possibleTemporaryErrorMaxRetries;
        this.possibleTemporaryErrorMsBeforeRetry = possibleTemporaryErrorMsBeforeRetry;
    }

    public int getTimeoutMaxRetries() {
        return timeoutMaxRetries;
    }

    public boolean isAutoRetryOnTimeout() {
        return timeoutMaxRetries > 0;
    }

    public int getBusyMaxRetries() {
        return busyMaxRetries;
    }

    public long getTimeoutMsBeforeRetry() {
        return timeoutMsBeforeRetry;
    }

    public long getBusyMsBeforeRetry() {
        return busyMsBeforeRetry;
    }

    public int getPossibleTemporaryErrorMaxRetries() {
        return possibleTemporaryErrorMaxRetries;
    }

    public long getPossibleTemporaryErrorMsBeforeRetry() {
        return possibleTemporaryErrorMsBeforeRetry;
    }

    @Override
    public String toString() {
        return "RetrySettings{" +
                "timeoutMaxRetries=" + timeoutMaxRetries +
                ", timeoutMsBeforeRetry=" + timeoutMsBeforeRetry +
                ", busyMaxRetries=" + busyMaxRetries +
                ", busyMsBeforeRetry=" + busyMsBeforeRetry +
                ", possibleTemporaryErrorMaxRetries=" + possibleTemporaryErrorMaxRetries +
                ", possibleTemporaryErrorMsBeforeRetry=" + possibleTemporaryErrorMsBeforeRetry +
                '}';
    }
}
