package be.iminds.ilabt.jfed.util;

import java.util.List;

/**
 * ExtraInfoCallback: This class is used whenever some function needs extra info.
 * It is a bit aof a HACK, which keeps separation between experimenter GUI (and other GUIS) and core library,
 * but does enable user feedback
 *
 * Supported feedback hacks:
 *   - bonfire username and password
 *   - federation API SA project to use
 */
public class ExtraInfoCallback {
    public static class NoBonfireAccountException extends RuntimeException { }

    public interface BonfireLoginCallback {
        String getBonfireGroups();
    }
    public interface AnyLoginCallback {
        /** return null if user has no account or provides none. */
        JFedPasswordManager.LoginInfo getLogin(String serviceName, String friendlyName);
    }
    public interface FederationApiProjectSelectionCallback {
        GeniUrn selectProject(List<GeniUrn> projectUrns);
    }

    private static BonfireLoginCallback bonfireLoginCallback;
    private static AnyLoginCallback anyLoginCallback;
    private static FederationApiProjectSelectionCallback federationApiProjectSelectionCallback;

    public static JFedPasswordManager.LoginInfo getAnyLogin(String serviceName, String friendlyName) {
        assert anyLoginCallback != null;
        return anyLoginCallback.getLogin(serviceName, friendlyName);
    }
    public static String getBonfireGroups() {
        assert bonfireLoginCallback != null;
        return bonfireLoginCallback.getBonfireGroups();
    }
    public static GeniUrn getFederationApiProjectSelection(List<GeniUrn> projectUrns) {
        assert federationApiProjectSelectionCallback != null;
        return federationApiProjectSelectionCallback.selectProject(projectUrns);
    }

    public static BonfireLoginCallback getBonfireLoginCallback() {
        return bonfireLoginCallback;
    }
    public static FederationApiProjectSelectionCallback getFederationApiProjectSelectionCallback() {
        return federationApiProjectSelectionCallback;
    }

    public static void setBonfireLoginCallback(BonfireLoginCallback bonfireLoginCallback) {
        ExtraInfoCallback.bonfireLoginCallback = bonfireLoginCallback;
    }
    public static void setAnyLoginCallback(AnyLoginCallback anyLoginCallback) {
        ExtraInfoCallback.anyLoginCallback = anyLoginCallback;
    }
    public static void setFederationApiProjectSelectionCallback(FederationApiProjectSelectionCallback federationApiProjectSelectionCallback) {
        ExtraInfoCallback.federationApiProjectSelectionCallback = federationApiProjectSelectionCallback;
    }
}
