package be.iminds.ilabt.jfed;

import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.ServerType;
import be.iminds.ilabt.jfed.lowlevel.TestCertHelper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.JFedTrustStore;
import be.iminds.ilabt.jfed.util.KeyUtil;
import com.google.inject.AbstractModule;
import com.google.inject.Provides;

import javax.annotation.Nullable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * User: twalcari
 * Date: 1/13/2015
 * Time: 11:40
 */
public class TestAuthoritiesModule extends AbstractModule {
    public static final String hrnA = "Example auth with user for testing";
    public static final String urlA_am2 = "https://am."+TestCertHelper.authority1Hostname+"/2.0";
    public static final String urlA_am3 = "https://am."+TestCertHelper.authority1Hostname+"/3.0";
    public static final String urlA_pgSa1 = "https://sa."+TestCertHelper.authority1Hostname+"/";

    public static final String hrnB = "Example 2nd auth for testing";
    public static final String urlB_am2 = "https://am."+TestCertHelper.authority2Hostname+"/2.0";
    public static final String urlB_am3 = "https://am."+TestCertHelper.authority2Hostname+"/3.0";
    public static final String urlB_pgSa1 = "https://sa."+TestCertHelper.authority2Hostname+"/";

    public static final String hrnC = "Example SCS for testing";
    public static final String urlC_scs = "https://scs."+TestCertHelper.authority3Hostname+"/scs";

    public static final String hrnFake = "Fake Authority for testing ... fake authorities";

    public static final String urlC_am2 = "https://am."+TestCertHelper.authority3Hostname+"/am/2.0";
    public static final String urlC_am3 = "https://am."+TestCertHelper.authority3Hostname+"/am/3.0";

    public static class AMConfig {
        private final boolean hasV2;
        private final boolean hasV3;
        private final boolean hasUpToDateAdv;
        private final boolean hasAny;

        public AMConfig() {
            this(false, true, false, true);
        }
        public AMConfig(boolean hasV2, boolean hasV3, boolean hasUpToDateAdv, boolean hasAny) {
            this.hasV2 = hasV2;
            this.hasV3 = hasV3;
            this.hasUpToDateAdv = hasUpToDateAdv;
            this.hasAny = hasAny;
        }
    }

    private final AMConfig am1conf;
    private final AMConfig am2conf;
    private final AMConfig am3conf;

    public TestAuthoritiesModule() {
        this(new AMConfig(), new TestAuthoritiesModule.AMConfig(true/*hasV2*/, false/*hasV3*/, false/*hasUpToDateAdv*/, false/*hasAny*/), new AMConfig());
    }
    public TestAuthoritiesModule(AMConfig am1conf, AMConfig am2conf, AMConfig am3conf) {
        this.am1conf = am1conf;
        this.am2conf = am2conf;
        this.am3conf = am3conf;
    }

    @Override
    protected void configure() {
//        bind(File.class).annotatedWith(Names.named("authoritiesFile")).toInstance(authoritiesFile);
    }

    @Provides
    private AuthorityListModel provideAuthorityListModel(JFedTrustStore trustStore, TestCertHelper testCertHelper) throws MalformedURLException, JFedException {
        AuthorityListModel authorityListModel = new AuthorityListModel();
//        boolean successFullLoad = StoredAuthorityList.load(this.authoritiesFile, authorityListModel);
//        if (!successFullLoad)
//            throw new RuntimeException("Failed to load test authority list from "+authoritiesFile);

        Map< ServerType, URL > urls = new HashMap<ServerType, URL>();
        if (am1conf.hasV2) urls.put(new ServerType(ServerType.GeniServerRole.AM, 2), new URL(urlA_am2));
        if (am1conf.hasV3) urls.put(new ServerType(ServerType.GeniServerRole.AM, 3), new URL(urlA_am3));
        urls.put(new ServerType(ServerType.GeniServerRole.PROTOGENI_SA, 1), new URL(urlA_pgSa1));
        SfaAuthority auth = new SfaAuthority(TestCertHelper.am1Urn, hrnA, null,urls,
                new ArrayList() /*proxies*/, null /*gid*/, "test"/*type*/);
        auth.addPemSslTrustCert(KeyUtil.x509certificateToPem(testCertHelper.authority1Cert));
        auth.setSpecialCase(SfaAuthority.SpecialCase.STITCHING_SUPPORTS_ANY, am1conf.hasAny);
        auth.setSpecialCase(SfaAuthority.SpecialCase.STITCHING_ADVERTISEMENT_RSPEC_UPTODATE, am1conf.hasUpToDateAdv);
        auth.setSpecialCase(SfaAuthority.SpecialCase.ADD_EXPIRES_ATTRIBUTE_TO_REQUEST, true);
        authorityListModel.addAuthority(auth);

        Map< ServerType, URL > urlsB = new HashMap<ServerType, URL>();
        if (am2conf.hasV2) urlsB.put(new ServerType(ServerType.GeniServerRole.AM, 2), new URL(urlB_am2));
        if (am2conf.hasV3) urlsB.put(new ServerType(ServerType.GeniServerRole.AM, 3), new URL(urlB_am3));
        urlsB.put(new ServerType(ServerType.GeniServerRole.PROTOGENI_SA, 1), new URL(urlB_pgSa1));
        SfaAuthority auth2 = new SfaAuthority(TestCertHelper.am2Urn, hrnB, null, urlsB,
                new ArrayList() /*proxies*/, null /*gid*/, "test"/*type*/);
        auth2.addPemSslTrustCert(KeyUtil.x509certificateToPem(testCertHelper.authority2Cert));
        auth2.setSpecialCase(SfaAuthority.SpecialCase.STITCHING_SUPPORTS_ANY, am2conf.hasAny);
        auth2.setSpecialCase(SfaAuthority.SpecialCase.STITCHING_ADVERTISEMENT_RSPEC_UPTODATE, am2conf.hasUpToDateAdv);
        auth2.setSpecialCase(SfaAuthority.SpecialCase.ADD_EXPIRES_ATTRIBUTE_TO_REQUEST, false);
        authorityListModel.addAuthority(auth2);

        Map< ServerType, URL > urlsC = new HashMap<ServerType, URL>();
        if (am3conf.hasV2) urlsC.put(new ServerType(ServerType.GeniServerRole.AM, 2), new URL(urlC_am2));
        if (am3conf.hasV3) urlsC.put(new ServerType(ServerType.GeniServerRole.AM, 3), new URL(urlC_am3));
        urlsC.put(new ServerType(ServerType.GeniServerRole.SCS, 1), new URL(urlC_scs));

        SfaAuthority auth3 = new SfaAuthority(TestCertHelper.scsUrn, hrnC, null, urlsC,
                new ArrayList() /*proxies*/, null /*gid*/, "test"/*type*/);
        auth3.addPemSslTrustCert(KeyUtil.x509certificateToPem(testCertHelper.authority3Cert));
        auth3.setSpecialCase(SfaAuthority.SpecialCase.STITCHING_SUPPORTS_ANY, am3conf.hasAny);
        auth3.setSpecialCase(SfaAuthority.SpecialCase.STITCHING_ADVERTISEMENT_RSPEC_UPTODATE, am3conf.hasUpToDateAdv);
        auth3.setSpecialCase(SfaAuthority.SpecialCase.ADD_EXPIRES_ATTRIBUTE_TO_REQUEST, true);
        authorityListModel.addAuthority(auth3);
        
        SfaAuthority fakeAuth= new SfaAuthority(TestCertHelper.fakeAmUrn, hrnFake, null, Collections.<ServerType, URL>emptyMap(),
                new ArrayList() /*proxies*/, null /*gid*/, "fake"/*type*/);
        authorityListModel.addAuthority(fakeAuth);

        trustStore.addAuthorityCerts(authorityListModel);
        return authorityListModel;
    }

    public String getTestAuthorityUrnString() {
        return TestCertHelper.am1Urn;
    }
    public String getTestAuthority2UrnString() {
        return TestCertHelper.am2Urn;
    }
    public String getTestAuthority3UrnString() {
        return TestCertHelper.am3Urn;
    }

    public GeniUrn getTestAuthorityUrn() {
        return GeniUrn.parse(getTestAuthorityUrnString());
    }
    public GeniUrn getTestAuthority2Urn() {
        return GeniUrn.parse(getTestAuthority2UrnString());
    }
    public GeniUrn getTestAuthority3Urn() {
        return GeniUrn.parse(getTestAuthority3UrnString());
    }
}
